<?php
namespace Wovnio\Wovnwp\WpClasses;

use Wovnio\Wovnwp\Constants;

global $wp_version;
if (defined('ABSPATH') && isset($wp_version) && version_compare($wp_version, '2.5', '>=')) {
	include_once(ABSPATH . 'wp-admin/includes/plugin.php');
}

class Swapper {
	private $accessed_url;
	private $options;
	private $request_variables;
	private $request_handler;
	private $target_lang;
	private $original_url;
	private $headers_transformer;

	public function __construct($accessed_url, $options, $request_variables, $request_handler, &$localization_decider, $headers_transformer = null) {
		$this->accessed_url = $accessed_url;
		$this->options = $options;
		$this->request_variables = $request_variables;
		$this->request_handler = $request_handler;
		$this->localization_decider = $localization_decider;
		$this->headers_transformer = $headers_transformer;
	}

	public function set_lang_and_original_url() {
		$changed_url = null;
		switch ($this->options['wovn_url_pattern_name']) {
			case 'query':
				$changed_url = $this->set_lang_and_original_url_by_query();
				break;
			case 'path':
				$changed_url = $this->set_lang_and_original_url_by_path();
				break;
			case 'subdomain':
				$changed_url = $this->set_lang_and_original_url_by_subdomain();
				break;
		}

		return $changed_url;
	}

	function set_lang_and_original_url_by_query() {
		$wovn_query = $this->request_variables["WOVN_QUERY"];
		if (isset($wovn_query) && in_array($wovn_query, $this->options['wovn_supported_langs'])) {
			$this->target_lang = $wovn_query;
			$this->original_url = $this->remove_query($this->accessed_url, $this->options['wovn_lang_param_name']);
		} else {
			$this->target_lang = $this->options['wovn_default_lang'];
			$this->original_url = $this->accessed_url;
		}

		return null;
	}

	function set_lang_and_original_url_by_path() {
		$uri = $this->request_variables['REQUEST_URI'];
		if (preg_match('/\/(' . join('|', $this->options['wovn_supported_langs']) .')(\/|$)/i', $uri, $matches)) {
			$this->target_lang = $matches[1];
			$lang_path_regexp = '/\/(' . join('|', $this->options['wovn_supported_langs']) .')(\/|$)/i';
			$this->original_url = preg_replace($lang_path_regexp, '/', $this->accessed_url, 1);
			return preg_replace('/\/' . $this->target_lang . '(\/|$)/i', '/', $uri, 1);
		} else {
			$this->target_lang = $this->options['wovn_default_lang'];
			$this->original_url = $this->accessed_url;
		}

		return null;
	}

	function set_lang_and_original_url_by_subdomain() {
		$uri = $this->request_variables['HTTP_HOST'];
		if (preg_match('/^(' . join('|', $this->options['wovn_supported_langs']) .')(\.)/', $uri, $matches)) {
			$this->target_lang = $matches[1];
			$lang_path_regexp = '/(' . join('|', $this->options['wovn_supported_langs']) .')(\.)/';
			$this->original_url = preg_replace($lang_path_regexp, '', $this->accessed_url, 1);
		} else {
			$this->target_lang = $this->options['wovn_default_lang'];
			$this->original_url = $this->accessed_url;
		}

		return null;
	}

	function remove_query($url, $lang_param_name) {
		$query_without_url = preg_replace('/([?&])'.$lang_param_name.'=[^&#]+(&|$)?(#)?/','$1$3',$url);
		return preg_replace('/(\?|&)($|#)/', '$2', $query_without_url);
	}

	public function validate() {
		if(!isset($this->target_lang) || !in_array($this->target_lang, $this->options['wovn_supported_langs'])) {
			$this->target_lang = $this->options['wovn_default_lang'];
		}
	}

	public function swap_html($body) {
		if (!($this->localization_decider->can_swap($this->request_variables))) {
			return $body;
		}

		if ($this->target_lang === $this->options['wovn_default_lang']) {
			$converter = new HtmlConverter($body, $this->original_url, $this->options, $this->target_lang);
			return $converter->insert_snippet_and_hreflang_tags(false);
		}

		$result_html = $this->request_wovn_api(
			$this->original_url,
			$this->options['wovn_user_token'],
			$this->target_lang,
			$this->options['wovn_url_pattern_name'],
			$this->options['wovn_lang_param_name'],
			$body
		);

		if (isset($result_html)) {
			return $result_html;
		} else {
			$converter = new HtmlConverter($body, $this->original_url, $this->options, $this->target_lang);
			return $converter->insert_snippet_and_hreflang_tags(true);
		}
	}

	function request_wovn_api($url, $token, $lang_code, $url_pattern, $lang_param_name, $body) {
		if (!$this->request_handler) {
			return null;
		}

		$request_url = $this->build_api_url($token, $lang_code, $body);
		$wp_debug = defined('WP_DEBUG') && true === WP_DEBUG;
		$data = array(
			'url' => $url,
			'token' => $token,
			'lang_code' => $lang_code,
			'url_pattern' => $url_pattern,
			'body' => $body,
			'debug_mode' => $wp_debug,
			'product' => Constants::$product_name,
			'version' => Constants::$product_version,
			'lang_param_name' => $lang_param_name
		);
		list($result, $headers, $error) = $this->request_handler->send_request(
			'POST',
			$request_url,
			$data,
			$this->options['wovn_translation_timeout'],
			$this->options['wovn_http_proxy']
		);

		if (!$result) {
			if ($error) {
				header("X-Wovn-Error: $error");
			}

			return null;
		}

		$result_json = json_decode($result, true);

		if ($this->headers_transformer) {
			$this->headers_transformer->transform_and_apply_api_headers($headers);
		}

		return $result_json['body'];
	}

	function build_api_url($token, $lang_code, $body) {
		$path = parse_url($this->original_url, PHP_URL_PATH);;
		$body_hash = md5($body);
		$setting = $this->options;
		ksort($setting);
		$settings_hash = md5(serialize($setting));
		$cache_key = rawurlencode("(token=$token&settings_hash=$settings_hash&body_hash=$body_hash&path=$path&lang=$lang_code)");

		$api_url = Constants::$wovn_api_url;
		return $api_url . '?cache_key=' . $cache_key;
	}
}
